import os
import logging
import requests
import pandas as pd
from PIL import Image, UnidentifiedImageError
from moviepy.editor import ImageSequenceClip, AudioFileClip
import numpy as np
from multiprocessing import Pool

def create_folders(districts):
    for district in districts:
        os.makedirs(os.path.join(district, 'male'), exist_ok=True)
        os.makedirs(os.path.join(district, 'female'), exist_ok=True)
        print(f"Created folders for {district} district")

def download_file(url, destination):
    if pd.notna(url) and isinstance(url, str) and url.startswith(('http://', 'https://')):
        response = requests.get(url)
        with open(destination, 'wb') as file:
            file.write(response.content)
        print(f"Downloaded file from {url} to {destination}")
    else:
        print(f"Skipping download for invalid URL or 'nan': {url}")

def create_video(image_path, audio_filename, output_folder, gender):
    try:
        # Check if the image path is not 'nan' and exists
        if pd.notna(image_path) and os.path.exists(image_path):
            # Resize image to a consistent size
            img = Image.open(image_path)

            if not img:
                return

            img = img.resize((640, 480), Image.BILINEAR)
            frame = np.array(img)

            # Create video with a single frame
            video_clip = ImageSequenceClip([frame], fps=1)

            # Set audio if available
            if os.path.exists(audio_filename):
                audio_clip = AudioFileClip(audio_filename)
                video_clip = video_clip.set_audio(audio_clip)

                # Set the duration of the video based on the length of the audio
                video_clip = video_clip.set_duration(audio_clip.duration)

                # Generate a unique video name using the image filename (excluding extension)
                video_name = os.path.splitext(os.path.basename(audio_filename))[0]
                video_name = f"{video_name}.mp4"

                # Write the video file to the respective folder inside the district
                output_folder_gender = os.path.join(output_folder, gender)
                os.makedirs(output_folder_gender, exist_ok=True)
                video_clip.write_videofile(os.path.join(output_folder_gender, video_name),
                                            codec='libx264', audio_codec='aac', fps=1, threads=4)

                print(f"Video created and saved to {output_folder_gender}/{video_name}")
                # Close the video clip to avoid issues
                video_clip.close()
            else:
                print(f"Skipping video creation for {output_folder} ({gender}) as the audio file path is 'nan' or does not exist.")
        else:
            print(f"Skipping video creation for {output_folder} ({gender}) as the image file path is 'nan' or does not exist.")

    except:
        print(f"Error processing")
        print(f"Skipping the current file and continuing with the next one.")


def process_excel(row):
        district_folder = row['District']
        male_folder = os.path.join(district_folder, 'male')
        female_folder = os.path.join(district_folder, 'female')

        if pd.notna(row['ImageFilename']) and pd.notna(row['AudioFilename']):
            male_image_filename = os.path.join(male_folder, os.path.basename(str(row['ImageFilename'])))
            male_audio_filename = os.path.join(male_folder, os.path.basename(str(row['AudioFilename'])))
            download_file(row['ImageFilename'], male_image_filename)
            download_file(row['AudioFilename'], male_audio_filename)
            create_video(male_image_filename, male_audio_filename, district_folder, 'male')

        if pd.notna(row['female image links']) and pd.notna(row['female audio links']):
            female_image_filename = os.path.join(female_folder, os.path.basename(str(row['female image links'])))
            female_audio_filename = os.path.join(female_folder, os.path.basename(str(row['female audio links'])))
            download_file(row['female image links'], female_image_filename)
            download_file(row['female audio links'], female_audio_filename)
            create_video(female_image_filename, female_audio_filename, district_folder, 'female')

def process_row(args):
    df, index, row = args
    print(f"Processing row {index + 1}/{len(df)}")
    process_excel(row)

if __name__ == "__main__":
    excel_path = r"/data2/data_nginx/videos/District_gender_speech_updated.xlsx"

    df = pd.read_excel(excel_path)
    unique_districts = set(df['District'])
    create_folders(unique_districts)

    # Use multiprocessing to process rows concurrently
    with Pool() as pool:
        pool.map(process_row, [(df, index, row[1]) for index, row in enumerate(df.iterrows())])
